# Return kinetic parameters, constraints, and vector of volumes in each
# compartment (pL) (1 if gating variable, or in element corresponding to
# kappa)

# Adapted from Pan 2018 cardiac AP 

import numpy as np

def kinetic_parameters(M, include_type2_reactions, dims, V):
    # Set the kinetic rate constants

    num_cols = dims['num_cols']
    num_rows = dims['num_rows']


    # constants are stored in V
    F = V['F']
    R = V['R']
    T = V['T']
    N_A = V['N_A']
    # G_GHK must be positive
    G_GHK = 1.111920573242084e-15    # Unit mA/mM
    P_nsCa = G_GHK/F * 1e12 # Unit pL/s . G_GHK [=] Amp/(mol/s)
    x_nsCa_channel = 4261 / N_A * 1e15 # unit    fmol

    # Calculate bond graph constants from kinetic parameters
    # Note: units of kappa are fmol/s, units of K are fmol^-1
    kf_nsCa = [P_nsCa/x_nsCa_channel]
    kr_nsCa = [P_nsCa/x_nsCa_channel]

    k_kinetic = kf_nsCa + kr_nsCa

    # CONSTRAINTS
    N_cT = []
    K_C = []

    # volume vector
    # W = list(np.append([1] * num_cols, [V['V_myo']] * num_rows))
    W = [1] * num_cols + [V['V_myo'], V['V_o']]

    return (k_kinetic, N_cT, K_C, W)













